package de.davidochmann.framework.dosite 
{
	/**
	 * @author dochmann
	 */
	import de.davidochmann.framework.dosite.event.DoSiteEvent;	
	import de.davidochmann.framework.dosite.event.SubsiteContainerEvent;	
	
	import flash.events.EventDispatcher;	
	import flash.display.Sprite;	
	import flash.display.Stage;	
	import flash.utils.getDefinitionByName;	
	import de.davidochmann.framework.dosite.framework.Framework;
	import de.davidochmann.framework.dosite.data.navigation.Navigation;	
	import de.davidochmann.framework.dosite.data.register.DisplayRegister;
	import de.davidochmann.framework.dosite.data.register.LoadingRegister;
	 
	public class DoSite extends EventDispatcher
	{
		private static var mInstance:DoSite;
		
		private var mStage:Stage;
		private var mXmlPath:String;
		private var mLoad:Boolean;
		
		private var mFramework:Framework;

		
		public function DoSite(pSingeltonEnforcer:SingeltonEnforcer){}

		public static function getInstance():DoSite
		{
			if(! mInstance)
				mInstance = new DoSite(new SingeltonEnforcer());
				
			return mInstance;
		}

		
		public function initialize(pStage:Stage, 
								   pXmlPath:String, 
								   pLoad:Boolean = true):void
		{
			setStage(pStage);
			setXmlPath(pXmlPath);
			setLoad(pLoad);
			
			initFramework();
			initEventHandler();
			
			if(mLoad)
				load();						
		}

		
		public function load():void
		{
			mFramework.load();
		}


		public function framework():Framework
		{
			return mFramework;
		}		
		
		public function container():Sprite
		{
			return mFramework.container();
		}

		public function display():DisplayRegister
		{
			return mFramework.displayRegister();
		}

		public function loading():LoadingRegister
		{
			return mFramework.loadingRegister();
		}
		
		public function current():String
		{
			return Navigation(mFramework.navigation()).current();
		}

		public function open(pId:String):void
		{
			Navigation(mFramework.navigation()).open(pId);
		}
		
		
		public function getStage():Stage
		{
			return mStage;
		}
		
		public function getXmlPath():String
		{
			return mXmlPath;
		}

		public function getLoad():Boolean
		{
			return mLoad;
		}


		public function setStage(pStage:Stage):void
		{
			mStage = pStage;
		}

		public function setXmlPath(pXmlPath:String):void
		{
			mXmlPath = pXmlPath;
		}

		public function setLoad(pLoad:Boolean):void
		{
			mLoad = pLoad;
		}

		
		private function initFramework():void
		{
			mFramework = (getDefinitionByName("de.davidochmann.framework.dosite.framework.Framework") as Class).getInstance();
			mFramework.initialize(mStage, mXmlPath);
		}
		
		private function initEventHandler():void
		{
			mFramework.navigation().addEventListener(SubsiteContainerEvent.OPEN, subsiteContainerOpenHandler);
		}
		
		
		private function subsiteContainerOpenHandler(e:SubsiteContainerEvent):void
		{
			dispatchEvent(new DoSiteEvent(DoSiteEvent.SITE_OPEN));			
		}
	}
}

class SingeltonEnforcer{}
