package de.davidochmann.framework.dosite.content
{
	/**
	 * @author dochmann
	 */
	
	import flash.events.Event;	
	import flash.events.EventDispatcher;	
	import de.davidochmann.framework.dosite.event.XmlReaderEvent;	

	public class XmlReader extends EventDispatcher
	{
		private static var mInstance:XmlReader;
		private static var DISPATCH_NUMBER:uint = 3; 
		
		private var mXmlPath:String;
		private var mEventCounter:uint;
		
		private var mXmlManagerDoSite:XmlManagerDoSite;
		private var mXmlManagerContent:XmlManagerContent;
		private var mXmlManagerComposition:XmlManagerComposition;


		public function XmlReader(pSingeltonEnforcer:SingeltonEnforcer){}
		
		public static function getInstance():XmlReader
		{
			if(! mInstance)
				mInstance = new XmlReader(new SingeltonEnforcer());
				
			return mInstance;
		}

		public function initialize(pXmlPath:String):void
		{
			setXmlPath(pXmlPath);

			initEventCounter();
			initXmlManagerDoSite();
			initXmlManagerContent();			
			initXmlManagerComposition();
		}
		
		
		public function getXmlPath():String
		{
			return mXmlPath;
		}

		public function setXmlPath(pXmlPath:String):void
		{
			mXmlPath = pXmlPath;
		}
		
		
		public function dosite():XmlManagerDoSite
		{
			return mXmlManagerDoSite;
		}

		public function content():XmlManagerContent
		{
			return mXmlManagerContent;
		}
		
		public function composition():XmlManagerComposition
		{
			return mXmlManagerComposition;
		}

		
		public function loadDoSiteXml():void
		{
			mXmlManagerDoSite.getReceiver().getUrlLoader().addEventListener(Event.COMPLETE, doSiteCompleteHandler);
			mXmlManagerDoSite.getReceiver().setPath(mXmlPath);
			mXmlManagerDoSite.getReceiver().load();
		}
		
		public function loadContentXml():void
		{
			var path:String = mXmlManagerDoSite.getSetup().content;
			
			mXmlManagerContent.getReceiver().getUrlLoader().addEventListener(Event.COMPLETE, contentCompleteHandler);
			mXmlManagerContent.getReceiver().setPath(path);
			mXmlManagerContent.getReceiver().load();	
		}

		public function loadCompositionXml():void
		{
			var path:String = mXmlManagerDoSite.getSetup().composition;
			
			mXmlManagerComposition.getReceiver().getUrlLoader().addEventListener(Event.COMPLETE, compositionCompleteHandler);
			mXmlManagerComposition.getReceiver().setPath(path);
			mXmlManagerComposition.getReceiver().load();			
		}


		private function doSiteCompleteHandler(e:Event):void
		{
			dispatchEvent(new XmlReaderEvent(XmlReaderEvent.DOSITE_COMPLETE));
			increaseEventCounter();
		}

		private function contentCompleteHandler(e:Event):void
		{
			dispatchEvent(new XmlReaderEvent(XmlReaderEvent.CONTENT_COMPLETE));			
			increaseEventCounter();
		}
		
		private function compositionCompleteHandler(e:Event):void
		{
			dispatchEvent(new XmlReaderEvent(XmlReaderEvent.COMPOSITION_COMPLETE));
			increaseEventCounter();
		}

		private function allComplete():void
		{
			dispatchEvent(new XmlReaderEvent(XmlReaderEvent.ALL_COMPLETE));			
		}
		

		private function increaseEventCounter():void
		{
			mEventCounter ++;
			
			if(mEventCounter >= DISPATCH_NUMBER)
				allComplete();
		}

		
		private function initEventCounter():void
		{
			mEventCounter = 0;			
		}		
		
		private function initXmlManagerDoSite():void
		{
			mXmlManagerDoSite = XmlManagerDoSite.getInstance();
			mXmlManagerDoSite.initialize();			
		}
	
		private function initXmlManagerContent():void
		{
			mXmlManagerContent = XmlManagerContent.getInstance();
			mXmlManagerContent.initialize();
		}

		private function initXmlManagerComposition():void
		{
			mXmlManagerComposition = XmlManagerComposition.getInstance();
			mXmlManagerComposition.initialize();				
		}
	}
}

class SingeltonEnforcer{}