package de.davidochmann.framework.dosite.data.display
{	
	/**
	 * @author MR.WUT4
	 */
	
	import flash.display.Stage;	
	import flash.display.Sprite;		  	
	import flash.display.DisplayObject;	
	import de.davidochmann.framework.dosite.data.DataManager;	
	import de.davidochmann.framework.dosite.interfaces.ISubsite;	
	import de.davidochmann.framework.dosite.interfaces.IContent;	
	import de.davidochmann.framework.dosite.event.ImageContainerEvent;	
	import de.davidochmann.framework.dosite.event.ContentContainerEvent;	
	import de.davidochmann.framework.dosite.event.SubsiteContainerEvent;	
	import de.davidochmann.framework.dosite.graphic.container.SubsiteContainer;	
	import de.davidochmann.framework.dosite.data.register.DisplayRegister;	 
	import de.davidochmann.framework.dosite.graphic.container.ContentContainer;	
	import de.davidochmann.framework.dosite.graphic.container.DisplayContainer;	
	import de.davidochmann.framework.dosite.graphic.container.ImageContainer;	
	 
	public class DisplayInitializer 
	{
		private static var mInstance:DisplayInitializer;
		private var mDataManager:DataManager;
		private var mDisplayRegister:DisplayRegister;
		private var mDisplayContainer:DisplayContainer;


		public function DisplayInitializer(pSingeltonEnforcer:SingeltonEnforcer){}
		
		public static function getInstance():DisplayInitializer
		{
			if(! mInstance)
				mInstance = new DisplayInitializer(new SingeltonEnforcer());
				
			return mInstance;
		}
		
		
		public function initialize():void
		{
			initDataManager();
			initDisplayRegister();
			initDisplayContainer();
		}
		
		
		public function imageComplete(e:ImageContainerEvent):void
		{
			var container:ImageContainer = ImageContainer(e.target);
			var image:DisplayObject 	 = DisplayObject(container.graphicLoader().content());
			
			var format:Object 			 = container.getFormat();			
			
			var id:String 				 = format.id;
			var depth:uint 				 = e.getDepth();
			
			image.x = format.x;
			image.y = format.y;
			
			mDisplayRegister.image().add(image, id);
			mDisplayContainer.graphic().removeChildAt(depth);
			mDisplayContainer.graphic().addChildAt(image, depth);
		}

		public function contentComplete(e:ContentContainerEvent):void
		{
			var container:ContentContainer = ContentContainer(e.target);
			var content:IContent		   = IContent(container.graphicLoader().content());
			
			var format:Object 			   = container.getFormat();
			var graphic:Object 			   = container.getGraphic();
			var extensible:Object		   = container.getExtensible();

			var id:String				   = format.id;			
			var width:uint				   = format.width;
			var height:uint				   = format.height;
			var depth:uint 				   = e.getDepth();
			var stage:Stage 			   = mDataManager.getStage();

			if(extensible != null)
				var path:String 		   = extensible.path;

			content.setStage(stage);
			content.setWidth(width);
			content.setHeight(height);
			content.setId(id);
			content.setPath(path);
			
			Sprite(content).x = format.x;
			Sprite(content).y = format.y;
			
			mDisplayRegister.content().add(content, id);
			mDisplayContainer.content().removeChildAt(depth);
			mDisplayContainer.content().addChildAt(Sprite(content), depth);

			content.initialize();
		}
	
		public function subsiteComplete(e:SubsiteContainerEvent):void
		{
			var startSite:String 		   = String(mDataManager.setup().search("type", "composition")[0].init);
			
			var container:SubsiteContainer = SubsiteContainer(e.target);
			var subsite:ISubsite 		   = ISubsite(container.graphicLoader().content());
			
			var format:Object 			   = container.getFormat();
			var extensible:Object		   = container.getExtensible();
	
			var id:String				   = format.id;		
			var title:String			   = format.title;
			var width:uint				   = format.width;
			var height:uint				   = format.height;
			var stage:Stage 			   = mDataManager.getStage();
			
			if(extensible != null)
				var path:String 		   = extensible.path;

			subsite.setStage(stage);
			subsite.setWidth(width);
			subsite.setHeight(height);
			subsite.setId(id);
			subsite.setTitle(title);
			subsite.setPath(path);
			
			Sprite(subsite).x = format.x;
			Sprite(subsite).y = format.y;
			
			mDisplayRegister.subsite().add(subsite, id);
			mDisplayContainer.subsite().setStartSite(startSite);
			
			subsite.initialize();
		}

		
		private function initDataManager():void
		{
			mDataManager = DataManager.getInstance();
		}

		private function initDisplayRegister():void
		{
			mDisplayRegister = DisplayRegister.getInstance();
		}

		private function initDisplayContainer():void
		{
			mDisplayContainer = DisplayContainer.getInstance();
		}
	}
}

class SingeltonEnforcer{}