package de.davidochmann.framework.dosite.data.display
{	
	/**
	 * @author MR.WUT4
	 */
	 
	import flash.display.Sprite;		
	import de.davidochmann.framework.dosite.graphic.container.DisplayContainer;	 
	import de.davidochmann.framework.dosite.data.DataManager;	 
	import de.davidochmann.framework.dosite.graphic.container.SubsiteContainer;	 
	import de.davidochmann.framework.dosite.graphic.container.ContentContainer;	 		
	import de.davidochmann.framework.dosite.graphic.container.ImageContainer;	

	public class DisplayManager 
	{
		private static var mInstance:DisplayManager;
		
		private var mDataManager:DataManager;
		private var mDisplayContainer:DisplayContainer;
		private var mImageList:Array;
		private var mContentList:Array;
		private var mSubsiteList:Array;
		
		private var mLoad:Boolean;		
		private var mInitialized:Boolean;
		
		
		public function DisplayManager(pSingeltonEnforcer:SingeltonEnforcer){}
		
		public static function getInstance():DisplayManager
		{
			if(! mInstance)
				mInstance = new DisplayManager(new SingeltonEnforcer());
				
			return mInstance;
		}
		
		public function initialize(pLoad:Boolean = false):void
		{
			setLoad(pLoad);
			setInitialized(true);		
			
			initDataManager();
			initDisplayContainer();
			initSubsiteList();				
			initImageList();
			initContentList();
			initSubsiteItems();
			initImageItems();
			initInteractiveItems();
		}
		
		
		public function imageList():Array
		{
			return mImageList;
		}		

		public function contentList():Array
		{
			return mContentList;
		}
		
		public function subsiteList():Array
		{
			return mSubsiteList;
		}


		public function load():void
		{
			loadSubsiteItems();
			loadImageItems();
			loadInteractiveItems();
		}
		
		
		public function getLoad():Boolean
		{
			return mLoad;
		}

		public function getInitialized():Boolean
		{
			return mInitialized;
		}


		public function setLoad(pLoad:Boolean):void
		{
			mLoad = pLoad;
		}
		
		public function setInitialized(pInitialized:Boolean):void
		{
			mInitialized = pInitialized;
		}

		
		private function loadImageItems():void
		{
			var length:uint = mImageList.length;
			var imageContainer:ImageContainer;
			
			for(var i:uint = 0; i < length; ++i)
			{
				imageContainer = ImageContainer(mImageList[i]);				
				imageContainer.graphicLoader().load(); 	
			}	
		}
		
		private function loadInteractiveItems():void
		{
			var length:uint = mContentList.length;
			var interactiveContainer:ContentContainer;
			
			for(var i:uint = 0; i < length; ++i)
			{
				interactiveContainer = ContentContainer(mImageList[i]);				
				interactiveContainer.graphicLoader().load(); 	
			}	
		}
		
		private function loadSubsiteItems():void
		{
			var length:uint = mSubsiteList.length;
			var subsiteContainer:SubsiteContainer;
			
			for(var i:uint = 0; i < length; ++i)
			{
				subsiteContainer = SubsiteContainer(mSubsiteList[i]);
				subsiteContainer.graphicLoader().load();
			}
		}

		
		private function initImageItems():void
		{
			var formatList:Array = mDataManager.composition().search("type", "img");
			var imageList:Array  = mDataManager.graphic().list();
			
			var sprite:Sprite	 = null;
			
			var format:Object	 = null;
			var graphic:Object	 = null;
			var depth:uint		 = undefined;
			var load:Boolean 	 = mLoad;
			
			var imageContainer:ImageContainer;
			
			for(var k:uint = 0; k < formatList.length; ++k)
			{
				sprite = new Sprite();
				mDisplayContainer.graphic().addChild(sprite);				
			}
			
			for(var i:uint = 0; i < formatList.length; ++i)
			{		
				format = formatList[i];	
				depth  = i;
				
				for(var j:uint = 0; j < imageList.length; j++)
				{
					graphic = imageList[j];
					
					if(graphic.id == format.id)
						break;						
				}		
				
				imageContainer = new ImageContainer(format, graphic, depth, load);
				mImageList.push(imageContainer);
			}	
		}

		private function initInteractiveItems():void
		{
			var formatList:Array 	 = mDataManager.composition().search("type", "con");
			var graphicList:Array  	 = mDataManager.graphic().list();
			var extensibleList:Array = mDataManager.extensible().list();
				
			var id:String			 = null;
			var sprite:Sprite		 = null;
			
			var format:Object		 = null;
			var graphic:Object		 = null;
			var extensible:Object	 = null;
			var depth:uint			 = undefined;			
			var load:Boolean 	     = mLoad;
			
			var contentContainer:ContentContainer;
			
			for(var l:uint = 0; l < formatList.length; ++l)
			{
				sprite = new Sprite();
				mDisplayContainer.content().addChild(sprite);				
			}
			
			for(var i:uint = 0; i < formatList.length; ++i)
			{
				id 	   	   = String(formatList[i].id);
				format 	   = formatList[i]; 
				
				depth 	   = i;
				graphic    = null;
				extensible = null;
				
				for(var j:uint = 0; j < graphicList.length; ++j)
				{
					if(graphicList[j].id == id)
					{
						graphic = graphicList[j];
						break;
					}
				}
				
				for(var k:uint = 0; k < extensibleList.length; ++k)
				{
					if(extensibleList[k].id == id)
					{
						extensible = extensibleList[k];
						break;
					}
				}
										
				contentContainer = new ContentContainer(format, graphic, extensible, depth, load);
				mContentList.push(contentContainer);
			}
		}

		private function initSubsiteItems():void
		{
			var formatList:Array 	 = mDataManager.composition().search("type", "sub");
			var graphicList:Array  	 = mDataManager.graphic().list();
			var extensibleList:Array = mDataManager.extensible().list();
				
			var id:String			 = null;
			
			var format:Object		 = null;
			var graphic:Object		 = null;
			var extensible:Object	 = null;
			var depth:uint			 = undefined;
			var load:Boolean 	     = mLoad;
			
			var subsiteContainer:SubsiteContainer;
					
			for(var i:uint = 0; i < formatList.length; ++i)
			{
				id 	   	   = String(formatList[i].id);
				format 	   = formatList[i]; 
				
				depth 	   = i;
				graphic    = null;
				extensible = null;
				
				for(var j:uint = 0; j < graphicList.length; ++j)
				{
					if(graphicList[j].id == id)
					{
						graphic = graphicList[j];
						break;
					}
				}
				
				for(var k:uint = 0; k < extensibleList.length; ++k)
				{
					if(extensibleList[k].id == id)
					{
						extensible = extensibleList[k];
						break;
					}
				}
												
				subsiteContainer = new SubsiteContainer(format, graphic, extensible, depth, load);
				mSubsiteList.push(subsiteContainer);
			}
		}
		

		private function initDataManager():void
		{
			mDataManager = DataManager.getInstance();
		}

		private function initDisplayContainer():void
		{
			mDisplayContainer = DisplayContainer.getInstance();
		}

		private function initImageList():void
		{
			mImageList = new Array();
		}
		
		private function initContentList():void
		{
			mContentList = new Array();
		}
		
		private function initSubsiteList():void
		{
			mSubsiteList = new Array();
		}
	}
}

class SingeltonEnforcer{}