package de.davidochmann.framework.dosite.data.navigation
{
	/**
	 * @author MR.WUT4
	 */
	
	import flash.events.EventDispatcher;	 
	import com.asual.swfaddress.SWFAddress;	
	import flash.errors.IllegalOperationError;	
	import com.asual.swfaddress.SWFAddressEvent;	
	import de.davidochmann.framework.dosite.data.DataManager;		
	import de.davidochmann.framework.dosite.interfaces.ISubsite;	
	import de.davidochmann.framework.dosite.event.SubsiteContainerEvent;	
	import de.davidochmann.framework.dosite.data.register.SubsiteRegister;	
	import de.davidochmann.framework.dosite.graphic.container.DisplayContainer;	

	public class Navigation extends EventDispatcher
	{
		private static var mInstance:Navigation;
		
		private var mDisplayContainer:DisplayContainer;
		private var mSubsiteRegister:SubsiteRegister;	
		private var mBaseId:String;
		private var mCurrentSubsitePath:String;


		public function Navigation(pSingeltonEnforcer:SingeltonEnforcer){}
		
		public static function getInstance():Navigation
		{
			if(! mInstance)
				mInstance = new Navigation(new SingeltonEnforcer());
				
			return mInstance;
		}
		
		
		public function initialize():void
		{
			initBaseId();
			initDisplayContainer();
			initSubsiteRegister();
			
			initSWFAdressEventHandler();
		}

		
		public function current():String
		{
			return mCurrentSubsitePath;
		}

		public function open(pId:String):void
		{
			var subsite:ISubsite = mSubsiteRegister.item(pId);
			
			if(subsite != null)
				SWFAddress.setValue(subsite.getId());
			else
				throw new IllegalOperationError("Subsite has not been initialized.");			
		}
		
		
		private function initBaseId():void
		{
			mBaseId = DataManager.getInstance().setup().search("type", "composition")[0].init;
		}

		private function initDisplayContainer():void
		{
			mDisplayContainer = DisplayContainer.getInstance();
		}
		
		private function initSubsiteRegister():void
		{
			mSubsiteRegister = SubsiteRegister.getInstance();
		}
		
		
		private function initSWFAdressEventHandler():void
		{
			SWFAddress.addEventListener(SWFAddressEvent.CHANGE, swfAddressChangeHandler);	
		}

		private function swfAddressChangeHandler(e:SWFAddressEvent):void
		{			
			openSubsite(e.value);
		}
		
		
		private function openSubsite(pPath:String):void
		{
			var nesting:Array    = pPath.split("/").slice(1);
	
			var id:String 		 = (nesting[0] == "") ? mBaseId : nesting[0];
			var subsite:ISubsite = mSubsiteRegister.item(id); 

			
			if(pPath == mCurrentSubsitePath)
				return;

			if(subsite != null)
				mDisplayContainer.subsite().change(subsite);
			else
				return;

			
			var nestsite:ISubsite = subsite; 
			var nestId:String;

			for(var i:uint = 1; i < nesting.length; ++i)
			{
				nestId 			  = nesting[i];
				nestsite 		  = nestsite.subsite(nestId);

				if(nestsite == null)
					return;
																			
				SWFAddress.setTitle(nestsite.getTitle());
			}


			mCurrentSubsitePath = pPath;

			dispatchEvent(new SubsiteContainerEvent(SubsiteContainerEvent.OPEN));
		}
	}
}

class SingeltonEnforcer{}