package de.davidochmann.framework.dosite.framework
{	
	/**
	 * @author dochmann
	 */

	import flash.display.Sprite;	
	import flash.display.Stage;	
	import flash.events.EventDispatcher;	
	import de.davidochmann.framework.dosite.data.navigation.Navigation;		
	import de.davidochmann.framework.dosite.data.DataManager;	 
	import de.davidochmann.framework.dosite.content.XmlReader;	
	import de.davidochmann.framework.dosite.event.FrameworkEvent;		
	import de.davidochmann.framework.dosite.data.display.DisplayManager;	
	import de.davidochmann.framework.dosite.data.display.DisplayInitializer;	
	import de.davidochmann.framework.dosite.event.ImageContainerEvent;	
	import de.davidochmann.framework.dosite.event.SubsiteContainerEvent;	
	import de.davidochmann.framework.dosite.event.ContentContainerEvent;	
	import de.davidochmann.framework.dosite.data.register.DisplayRegister;	
	import de.davidochmann.framework.dosite.data.register.LoadingRegister;	
	import de.davidochmann.framework.dosite.graphic.container.ImageContainer;	
	import de.davidochmann.framework.dosite.graphic.container.DisplayContainer;	 	
	import de.davidochmann.framework.dosite.graphic.container.SubsiteContainer;	
	import de.davidochmann.framework.dosite.graphic.container.ContentContainer;	
	
	public class Framework extends EventDispatcher
	{
		private static var mInstance:Framework;
		private var mStage:Stage;
		private var mXmlPath:String;
		private var mLoad:Boolean;
		
		private var mXmlReader:XmlReader;
		private var mDataManager:DataManager;
		private var mDisplayContainer:DisplayContainer;
		private var mDisplayInitializer:DisplayInitializer;
		private var mDisplayManager:DisplayManager;
		private var mDisplayRegister:DisplayRegister;
		private var mNavigation:Navigation;
		private var mLoadingRegister:LoadingRegister;
		private var mFrameworkEventHandler:FrameworkEventHandler;


		public function Framework(pSingeltonEnforcer:SingeltonEnforcer){}
		
		public static function getInstance():Framework
		{
			if(! mInstance)
				mInstance = new Framework(new SingeltonEnforcer());
				
			return mInstance;
		}

		public function initialize(pStage:Stage, pXmlPath:String):void
		{
			setStage(pStage);
			setXmlPath(pXmlPath);
						
			initDataManager();
			initXmlReader();
			initDisplayContainer();
			initDisplayInitializer();
			initDisplayManager();
			initDisplayRegister();
			initNavigation();
			initLoadingRegister();
		
			initFrameworkEventHandler();
		}
		
		
		public function load():void
		{
			var init:Boolean = mDisplayManager.getInitialized();

			switch(init)
			{
				case true:
					mDisplayManager.load();
					break;
					
				case false:
					mLoad = true;
					break;	
			}
		}

		
		public function container():Sprite
		{
			return mDisplayContainer.container();
		}

		public function navigation():Navigation
		{
			return mNavigation;
		}

		public function displayRegister():DisplayRegister
		{
			return mDisplayRegister;
		}

		public function loadingRegister():LoadingRegister
		{
			return mLoadingRegister;
		}

		public function dataManager():DataManager
		{
			return mDataManager;
		}

		public function xmlReader():XmlReader
		{
			return mXmlReader;
		}
		
		public function displayContainer():DisplayContainer
		{
			return mDisplayContainer;
		}

		public function displayManager():DisplayManager
		{
			return mDisplayManager;
		}

				
		public function getStage():Stage
		{
			return mStage;
		}

		public function getXmlPath():String
		{
			return mXmlPath;
		}

		public function getLoad():Boolean
		{
			return mLoad;
		}

		
		public function setXmlPath(pXmlPath:String):void
		{
			mXmlPath = pXmlPath;
		}

		public function setStage(pStage:Stage):void
		{
			mStage = pStage;
		}

		public function setLoad(pLoad:Boolean):void
		{
			mLoad = pLoad;
		}

		
		public function buildSetupList():void
		{
			var dosite:Object 	   = mXmlReader.dosite().getSetup();
			var content:Object 	   = mXmlReader.content().getSetup();
			var composition:Object = mXmlReader.composition().getSetup();
						
			mDataManager.setup().add(dosite);
			mDataManager.setup().add(content);
			mDataManager.setup().add(composition);
		}

		public function buildExtensibleList():void
		{
			var length:uint = mXmlReader.content().getExtensibleLength();
			var extensible:Object;
			
			for(var i:uint = 0; i < length; ++i)
			{
				extensible = mXmlReader.content().getExtensible(i);
				mDataManager.extensible().add(extensible);
			}				
		}

		public function buildGraphicList():void
		{
			var length:uint = mXmlReader.content().getGraphicLength();
			var content:Object;
			
			for(var i:uint = 0; i < length; ++i)
			{
				content = mXmlReader.content().getGraphic(i);
				mDataManager.graphic().add(content);
			}
		}

		public function buildCompositionList():void
		{
			var lenght:uint = mXmlReader.composition().getFormationLength();
			var formation:Object;
			
			for(var i:uint = 0; i < lenght; ++i)
			{
				formation = mXmlReader.composition().getFormation(i);
				mDataManager.composition().add(formation);
			}
		}

		public function buildDisplayElements():void
		{
			mDisplayManager.initialize(mLoad);
		}

		public function initContainerPosition():void
		{
			mDisplayContainer.initContainerPosition();
		}

		public function initDisplayElements():void
		{	
			var subsiteList:Array = mDisplayManager.subsiteList();
			var subsite:SubsiteContainer;
			
			for(var k:uint = 0; k < subsiteList.length; ++k)
			{
				subsite = subsiteList[k];

				mLoadingRegister.addSubsiteContainer(subsite);		

				subsite.addEventListener(SubsiteContainerEvent.LOAD_COMPLETE, mDisplayInitializer.subsiteComplete);
				subsite.addEventListener(SubsiteContainerEvent.LOAD_COMPLETE, mLoadingRegister.subsiteComplete);				
			}

			var contentList:Array = mDisplayManager.contentList();
			var content:ContentContainer;
			
			for(var j:uint = 0; j < contentList.length; ++j)
			{
				content = contentList[j];
				
				mLoadingRegister.addContentContainer(content);
				
				content.addEventListener(ContentContainerEvent.LOAD_COMPLETE, mLoadingRegister.contentComplete);
				content.addEventListener(ContentContainerEvent.LOAD_COMPLETE, mDisplayInitializer.contentComplete);	
			}

			var imageList:Array = mDisplayManager.imageList();
			var image:ImageContainer;
			
			for(var i:uint = 0; i < imageList.length; ++i)
			{
				image = imageList[i];
				
				mLoadingRegister.addImageContainer(image);
				
				image.addEventListener(ImageContainerEvent.LOAD_COMPLETE, mLoadingRegister.imageComplete);
				image.addEventListener(ImageContainerEvent.LOAD_COMPLETE, mDisplayInitializer.imageComplete);	
			}
		
			dispatchEvent(new FrameworkEvent(FrameworkEvent.DISPLAY_ITEMS_ADDED));
		}

		public function allowNavigation():void
		{
			mNavigation.initialize();
		}

		
		private function initDataManager():void
		{
			mDataManager = DataManager.getInstance();
			mDataManager.initialize(mStage);
		}	

		private function initXmlReader():void
		{
			mXmlReader = XmlReader.getInstance();
			mXmlReader.initialize(mXmlPath);
			mXmlReader.loadDoSiteXml();		
		}

		private function initDisplayContainer():void
		{
			mDisplayContainer = DisplayContainer.getInstance();
			mDisplayContainer.initialize();
		}

		private function initDisplayInitializer():void
		{
			mDisplayInitializer = DisplayInitializer.getInstance();
			mDisplayInitializer.initialize();
		}

		private function initDisplayManager():void
		{
			mDisplayManager = DisplayManager.getInstance();
		}

		private function initDisplayRegister():void
		{
			mDisplayRegister = DisplayRegister.getInstance();
			mDisplayRegister.initialize();
		}

		private function initNavigation():void
		{
			mNavigation = Navigation.getInstance();
		}

		private function initLoadingRegister():void
		{
			mLoadingRegister = LoadingRegister.getInstance();
			mLoadingRegister.initialize();
		}

		private function initFrameworkEventHandler():void
		{
			mFrameworkEventHandler = FrameworkEventHandler.getInstance();
			mFrameworkEventHandler.initialize();
		}
	}
}

class SingeltonEnforcer{}
