package de.davidochmann.framework.dosite.graphic.base
{	
	/**
	 * @author MR.WUT4
	 */
	 
	import flash.net.URLRequest;	
	import flash.geom.Point;	
	import flash.events.Event;	
	import flash.display.Loader;	
	import flash.display.Sprite;
	import flash.display.Graphics;	
	import flash.display.DisplayObject;	
	import flash.system.ApplicationDomain;	
	import flash.system.LoaderContext;	

	public class GraphicContainer extends Sprite 
	{
		private var mName:String;
		private var mPath:String;
		private var mLoad:Boolean;
		private var mWidth:uint;
		private var mHeight:uint;
		private var mColor:uint;
		private var mContentPoint:Point;
		private var mMaskPoint:Point;

		private var mLoader:Loader;
		private var mMask:Sprite;
		private var mBackground:Sprite;
		private var mContainer:Sprite;
		
		
		public function GraphicContainer(pName:String,
										 pPath:String,
										 pLoad:Boolean		 = true,
										 pWidth:uint   		 = 0,
										 pHeight:uint		 = 0,
										 pColor:uint   		 = 0x000000,
										 pContentPoint:Point = null,
										 pMaskPoint:Point    = null)
		{
			setName(pName);
			setPath(pPath);
			setLoad(pLoad);
			setWidth(pWidth);
			setHeight(pHeight);
			setColor(pColor);
			setContentPoint(pContentPoint);
			setMaskPoint(pMaskPoint);

			initLoader();
			initMask();
			initBackground();
			initContainer();

			if(mLoad)				
				load();
		}
		
		
		public function load():void
		{			
			mLoader.load(new URLRequest(mPath), new LoaderContext(false, ApplicationDomain.currentDomain));
		}

		public function unload():void
		{
			mLoader.unload();
		}
				
		public function content():DisplayObject
		{
			return mLoader.content;
		}
		
		public function loader():Loader
		{
			return mLoader;
		}

		public function getName():String
		{
			return mName;
		}
		
		public function getPath():String
		{
			return mPath;
		}
		
		public function getLoad():Boolean
		{
			return mLoad;
		}

		public function getWidth():uint
		{
			return mWidth;
		}

		public function getHeight():uint
		{
			return mHeight; 
		}

		public function getColor():uint
		{
			return mColor;
		}

		public function getContentPoint():Point
		{
			return mContentPoint;
		}

		public function getMaskPoint():Point
		{
			return mMaskPoint;
		}

		
		public function setName(pName:String):void
		{
			mName = pName;
		}

		public function setPath(pPath:String):void
		{
			mPath = pPath;
		}

		public function setLoad(pLoad:Boolean):void
		{
			mLoad = pLoad;
		}

		public function setWidth(pWidth:uint):void
		{
			mWidth = pWidth;
		}

		public function setHeight(pHeight:uint):void
		{
			mHeight = pHeight;
		}

		public function setColor(pColor:uint):void
		{
			mColor = pColor;
		}

		public function setContentPoint(pContentPoint:Point):void
		{
			mContentPoint = pContentPoint;
		}

		public function setMaskPoint(pMaskPoint:Point):void
		{
			mMaskPoint = pMaskPoint;
		}

		
		private function initLoader():void
		{
			mLoader = new Loader();		
			mLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, loaderCompleteHandler); 
		}

		private function initMask():void
		{
			mMask = new Sprite();
			mMask.cacheAsBitmap = true;
		}

		private function initBackground():void
		{
			mBackground = new Sprite();
			mBackground.cacheAsBitmap = true;
		}

		private function initContainer():void
		{
			mContainer = new Sprite();
			mContainer.cacheAsBitmap = true;
			
			mContainer.addChild(mBackground);
			mContainer.addChild(mLoader);
			mContainer.mask = mMask;
		}
		
		
		private function loaderCompleteHandler(e:Event):void
		{			
			draw(mMask      , 0x000000);
			draw(mBackground, mColor  );
			
			position(mMask          , mMaskPoint   );
			position(mLoader.content, mContentPoint);
			
			addChildren();
		}

		private function draw(pSprite:Sprite, pColor:uint):void
		{
			var canvas:Graphics = pSprite.graphics;
			var spriteWidth:uint;
			var spriteHeight:uint;
			
			if(mWidth != 0 && mHeight != 0)
			{
				spriteWidth  = mWidth;
				spriteHeight = mHeight;
			}
			else
			{				
				spriteWidth  = mLoader.content.width;
				spriteHeight = mLoader.content.height;
			}
						
			canvas.beginFill(pColor);
			canvas.drawRect(0, 0, spriteWidth, spriteHeight);
			canvas.endFill();
		}

		private function position(pDisplayObject:DisplayObject, pPoint:Point):void
		{
			if(pPoint != null)
			{
				pDisplayObject.x = pPoint.x;
				pDisplayObject.y = pPoint.y;
			}
		}

		
		private function addChildren():void
		{
			addChild(mMask);
			addChild(mContainer);			
		}
	}
}
